part of 'exercise_new_bloc.dart';

@immutable
abstract class ExerciseNewState extends Equatable {
  const ExerciseNewState();

  @override
  List<Object> get props => [];
}

class ExerciseNewInitial extends ExerciseNewState {
  const ExerciseNewInitial();
}

class ExerciseNewLoading extends ExerciseNewState {
  const ExerciseNewLoading();
}

class ExerciseNewReady extends ExerciseNewState {
  const ExerciseNewReady();
}

class ExerciseNewSaved extends ExerciseNewState {
  const ExerciseNewSaved();
}

class ExerciseNewError extends ExerciseNewState {
  final String message;
  const ExerciseNewError({required this.message});

  @override
  List<Object> get props => [message];
}
